# .SYNOPSIS
# Sync-MailPublicFoldersCloudToOnprem.ps1
#    This script imports the new mail public folders as sync mail public folders from Exchange Online to on-premise.
#	 And also synchronizes the properites of existing mail-enabled public folders from Exchange Online to on-premises (thereby overriding the mail public folder properties in on-premise).
#
# Example input to the script:
#
# Sync-MailPublicFoldersCloudToOnprem.ps1 -ConnectionUri <cloud url> -CsvSummaryFile <path for the summary file>
#
# The above example imports new mail public folders objects from Exchange Online as sync mail public folders to on-premise.
#
# .DESCRIPTION
#
# Copyright (c) 2016 Microsoft Corporation. All rights reserved.
#
# THIS CODE IS MADE AVAILABLE AS IS, WITHOUT WARRANTY OF ANY KIND. THE ENTIRE RISK
# OF THE USE OR THE RESULTS FROM THE USE OF THIS CODE REMAINS WITH THE USER.
param (
    [Parameter(Mandatory=$false)]
    [PSCredential] $Credential,

    [Parameter(Mandatory = $false)]
    [ValidateNotNull()]
    [string] $ConnectionUri = "https://outlook.office365.com/powerShell-liveID",

    [Parameter(Mandatory=$true)]
    [ValidateNotNullOrEmpty()]
    [string] $CsvSummaryFile
    )

# Writes a dated information message to console
function WriteInfoMessage()
{
    param ($message)
    Write-Host "[$($(Get-Date).ToString())]" $message;
}

# Writes an error importing a mail public folder to the CSV summary
function WriteErrorSummary()
{
    param ($folder, $operation, $errorMessage, $commandtext)

    WriteOperationSummary $folder.Guid $operation $errorMessage $commandtext;
    $script:errorsEncountered++;
}

# Writes the operation executed and its result to the output CSV
function WriteOperationSummary()
{
    param ($folder, $operation, $result, $commandtext)

    $columns = @(
        (Get-Date).ToString(),
        $folder.Guid,
        $operation,
        (EscapeCsvColumn $result),
        (EscapeCsvColumn $commandtext)
    );

    Add-Content $CsvSummaryFile -Value ("{0},{1},{2},{3},{4}" -f $columns);
}

#Escapes a column value based on RFC 4180 (http://tools.ietf.org/html/rfc4180)
function EscapeCsvColumn()
{
    param ([string]$text)

    if ($text -eq $null)
    {
        return $text;
    }

    $hasSpecial = $false;
    for ($i=0; $i -lt $text.Length; $i++)
    {
        $c = $text[$i];
        if ($c -eq $script:csvEscapeChar -or
            $c -eq $script:csvFieldDelimiter -or
            $script:csvSpecialChars -contains $c)
        {
            $hasSpecial = $true;
            break;
        }
    }

    if (-not $hasSpecial)
    {
        return $text;
    }
    
    $ch = $script:csvEscapeChar.ToString([System.Globalization.CultureInfo]::InvariantCulture);
    return $ch + $text.Replace($ch, $ch + $ch) + $ch;
}

## Create a tenant PSSession against Exchange Online using modern auth.
function InitializeExchangeOnlineRemoteSession()
{
	WriteInfoMessage $LocalizedStrings.CreatingRemoteSession;

    $oldWarningPreference = $WarningPreference;
    $oldVerbosePreference = $VerbosePreference;

    try
	{
		Import-Module ExchangeOnlineManagement -ErrorAction SilentlyContinue;
		if (Get-Module ExchangeOnlineManagement)
		{
			$sessionOption = (New-PSSessionOption -SkipCACheck);
			Connect-ExchangeOnline -Credential $Credential -ConnectionURI $ConnectionUri -PSSessionOption $sessionOption -Prefix "Remote" -ErrorAction SilentlyContinue;
			$script:isConnectedToExchangeOnline = $true;
		}
		else
		{
			Write-Warning $LocalizedStrings.EXOV2ModuleNotInstalled;
			Exit;
		}
	}
    catch
    {
       Write-Error "Error message: $($_.Exception.Message)";
       WriteInfoMessage ($LocalizedStrings.ConnectExchangeOnlineFailure);
       Exit;
    }
    WriteInfoMessage ($LocalizedStrings.RemoteSessionCreatedSuccessfully);
}

## Formats the command and its parameters to be printed on console or to file
function FormatCommand()
{
    param ([string]$command, [System.Collections.IDictionary]$parameters)

    $commandText = New-Object System.Text.StringBuilder;
    [void]$commandText.Append($command);
    foreach ($name in $parameters.Keys)
    {
        [void]$commandText.AppendFormat(" -{0}:",$name);

        $value = $parameters[$name];
        if ($value -isnot [Array])
        {
            [void]$commandText.AppendFormat("`"{0}`"", $value);
        }
        elseif ($value.Length -eq 0)
        {
            [void]$commandText.Append("@()");
        }
        else
        {
            [void]$commandText.Append("@(");
            foreach ($subValue in $value)
            {
                [void]$commandText.AppendFormat("`"{0}`",",$subValue);
            }
            
            [void]$commandText.Remove($commandText.Length - 1, 1);
            [void]$commandText.Append(")");
        }
    }

    return $commandText.ToString();
}

## Get external email address domains which are not configured on the on-premise connector
function GetMissingDomainsFromConnector()
{
    param($sendConnectorDomains)

    $missingDomains = @();

    foreach ($domain in $script:ExternalEmailAddressDomains.Keys)
    {
        if (-not $sendConnectorDomains.ContainsKey($domain))
        {
            $missingDomains += $domain;
        }
    }

    return $missingDomains;
}

## Check if the centralized transport feature is enabled
function IsCentralizedTransportEnabled()
{
    $hybridConf = Get-HybridConfiguration

    if ($hybridConf -ne $null -and $hybridConf.Features -ne $null -and $hybridConf.Features.Contains("CentralizedTransport"))
    {
        return $true;
    }

    return $false;
}

## Get send connector (on-premise) domains configured for O365
function GetSendConnectorDomains()
{
    $connector = Get-SendConnector -Identity $script:SendConnectorToO365 -ErrorAction:SilentlyContinue -WarningAction:SilentlyContinue;
    $domains = @{};

    if ($connector -ne $null -and $connector.AddressSpaces -ne $null)
    {
        foreach ($addressSpace in $connector.AddressSpaces)
        {
            $domain = $addressSpace.Address.ToString();

            if(-not $domains.ContainsKey($domain))
            {
                $domains.Add($domain, $false);
            }
        }
    }

    return $domains;
}

## Store distinct domains used in the external email addresses
function StoreExternalEmailAddressDomain(
    [string] $emailAddress)
{
    if (-not [string]::IsNullOrEmpty($emailAddress))
    {
        $index = $emailAddress.IndexOf('@')
        $domain = $emailAddress.Substring($index + 1)

        if (-not $script:ExternalEmailAddressDomains.ContainsKey($domain))
        {
            $script:ExternalEmailAddressDomains.Add($domain, $false)
        }
    }
}

## Concatenate a list of domains
function ConcatDomains()
{
    param($domainList)

    $domains = $script:separator;
    $domains += $domainList -join $script:separator
    return $domains;
}

## Get external email address from an EXO mail public folder
function GetExternalEmailAddress()
{
    param ($remotePublicFolder)

    $primarySmtpAddress = $remotePublicFolder.PrimarySmtpAddress.ToString();

    if ($primarySmtpAddress.EndsWith($script:OnMicrosoftDomain, [StringComparison]::InvariantCultureIgnoreCase))
    {
        return $primarySmtpAddress;
    }

    $externalEmailAddress = $primarySmtpAddress;
    $primarySmtpAddressParts = $primarySmtpAddress.Split($script:proxyAddressSeparators); # alias@domain
    
    if ($remotePublicFolder.EmailAddresses -ne $null -and $remotePublicFolder.EmailAddresses.Count -gt 0)
    {
        foreach ($address in $remotePublicFolder.EmailAddresses)
        {
            if ($address.StartsWith($script:SmtpPrefix, [StringComparison]::InvariantCultureIgnoreCase))
            {
                $addressParts = $address.Split($script:proxyAddressSeparators); # smtp:alias@domain

                if ($addressParts.Count -eq 3 -and
                    $addressParts[1].Equals($primarySmtpAddressParts[0], [StringComparison]::InvariantCultureIgnoreCase) -and 
                    $addressParts[2].EndsWith($script:OnMicrosoftDomain, [StringComparison]::InvariantCultureIgnoreCase))
                {
                    return (RemoveSmtpPrefix $address);
                }
            }
        }
    }

    return $externalEmailAddress;
}

## Remove smtp prefix from the email address
function RemoveSmtpPrefix()
{
    param ($emailAddress)

    if ([String]::IsNullOrEmpty($emailAddress))
    {
        return [String]::Empty;
    }

    if ($emailAddress.StartsWith($script:SmtpPrefix, [StringComparison]::InvariantCultureIgnoreCase))
    {
        return $emailAddress.Substring($script:SmtpPrefixLength);
    }

    return $emailAddress;
}

## Retrieve mail enabled public folders from EXO
function GetRemoteMailPublicFolders()
{    
    $mailPublicFolders = Get-RemoteMailPublicFolder -ResultSize:Unlimited -ErrorAction:SilentlyContinue -WarningAction:SilentlyContinue;
    
    # Return the results
    if ($mailPublicFolders -eq $null -or ([array]($mailPublicFolders)).Count -eq 0)
    {
		return $null;
	}
	
    return $mailPublicFolders;
}

#Get list of MEPF's in OnPrem to be deleted
function GetFoldersToMailDisable(
				[object[]] $localMailPublicFolders, 
				[hashtable] $validExternalEmailAddresses)
{
	$foldersToMailDisable = @();
	foreach($syncPublicFolder in $localMailPublicFolders)
    {
		$localExternalEmailAddress = [String]::Empty;

        if ($syncPublicFolder.ExternalEmailAddress -ne $null)
        {
            $localExternalEmailAddress = RemoveSmtpPrefix $syncPublicFolder.ExternalEmailAddress.ToString();
        }

		if (-not $validExternalEmailAddresses.ContainsKey($localExternalEmailAddress))
        {
			$foldersToMailDisable += $syncPublicFolder.Identity;
		}
	}
	return $foldersToMailDisable
}

## Sync mail public folders from cloud to on-premise.
function SyncMailPublicFolders(
    [object[]] $mailPublicFolders)
{
    $validExternalEmailAddresses = @{};

    if ($mailPublicFolders -ne $null)
    {        
        foreach ($mailPublicFolder in $mailPublicFolders)
        {
            # extracting properties
            $alias = $mailPublicFolder.Alias.Trim();
            $identity = $mailPublicFolder.PrimarySmtpAddress.ToString();
            $externalEmailAddress = GetExternalEmailAddress $mailPublicFolder;
            $entryId = $mailPublicFolder.EntryId.ToString();
            $name = $mailPublicFolder.Name.Trim();
            $displayName = $mailPublicFolder.DisplayName.Trim()
            $hiddenFromAddressListsEnabled = $mailPublicFolder.HiddenFromAddressListsEnabled;

            $windowsEmailAddress = $mailPublicFolder.WindowsEmailAddress.ToString();
            if ($windowsEmailAddress -eq "")
            {
                $windowsEmailAddress = $externalEmailAddress;
            }

            # extracting all the EmailAddress
            $emailAddress = @();
            foreach($address in $mailPublicFolder.EmailAddresses)
            {
                $emailAddress += $address.ToString();
            }

            # preserve the ability to reply via Outlook's nickname cache post-migration
            $emailAddress +=  ("X500:" + $mailPublicFolder.LegacyExchangeDN);

            WriteInfoMessage ($LocalizedStrings.SyncingMailPublicFolder -f $alias);

            $syncPublicFolder = Get-MailPublicFolder -Identity $identity -ErrorAction:SilentlyContinue -WarningAction:SilentlyContinue;
			
            if ($syncPublicFolder -eq $null)
            {
                WriteInfoMessage ($LocalizedStrings.CreatingSyncMailPublicFolder -f $alias);
                try
                {
                    $newParams = @{};
                    $newParams.Add("Name", $name);
                    $newParams.Add("ExternalEmailAddress", $externalEmailAddress);
                    $newParams.Add("Alias", $alias);
                    $newParams.Add("EntryId", $entryId);
                    $newParams.Add("WindowsEmailAddress", $windowsEmailAddress);
                    $newParams.Add("WarningAction", "SilentlyContinue");
                    $newParams.Add("ErrorAction", "Stop");

                    [string]$createSyncPublicFolder = (FormatCommand $script:NewSyncMailPublicFolderCommand $newParams);
                    
                    # Creating new sync mail public folder
                    $newMailPublicFolder = &$script:NewSyncMailPublicFolderCommand @newParams;

                    WriteOperationSummary $mailPublicFolder $LocalizedStrings.CreateOperationName $LocalizedStrings.CsvSuccessResult $createSyncPublicFolder;

                    $setParams = @{};
                    $setParams.Add("Identity", $name);
                    $setParams.Add("EmailAddresses", $emailAddress);
                    $setParams.Add("DisplayName", $displayName);
                    $setParams.Add("HiddenFromAddressListsEnabled", $hiddenFromAddressListsEnabled);
                    $setParams.Add("WarningAction", "SilentlyContinue");
                    $setParams.Add("ErrorAction", "Stop");

                    [string]$setOtherProperties = (FormatCommand $script:SetMailPublicFolderCommand $setParams);

                    # Setting other properties to the newly created sync mail public folder
                    &$script:SetMailPublicFolderCommand @setParams;
                    
                    WriteOperationSummary $mailPublicFolder $LocalizedStrings.SetOperationName $LocalizedStrings.CsvSuccessResult $setOtherProperties;

                    $validExternalEmailAddresses.Add($externalEmailAddress, $false);
                    $script:CreatedPublicFoldersCount++;
                }

                catch
                { 
                    WriteErrorSummary $mailPublicFolder $LocalizedStrings.CreateOperationName $_.Exception.Message "";
                    Write-Error $_;
                }
            }

            else
            {
                WriteInfoMessage ($LocalizedStrings.UpdatingSyncMailPublicFolder -f $syncPublicFolder);
                try
                {
                    $updateParams = @{};
                    $updateParams.Add("Identity", $syncPublicFolder);
                    $updateParams.Add("EmailAddresses", $emailAddress);
                    $updateParams.Add("HiddenFromAddressListsEnabled", $hiddenFromAddressListsEnabled);
                    $updateParams.Add("DisplayName", $displayName);
                    $updateParams.Add("Name", $name);
                    $updateParams.Add("ExternalEmailAddress", $externalEmailAddress);
                    $updateParams.Add("Alias", $alias);
                    $updateParams.Add("WindowsEmailAddress", $windowsEmailAddress);
                    $updateParams.Add("WarningAction", "SilentlyContinue");
                    $updateParams.Add("ErrorAction", "Stop");

                    [string]$updateProperties = (FormatCommand $script:SetMailPublicFolderCommand $updateParams);

                    # Setting properties to the existing sync mail public folder
                    &$script:SetMailPublicFolderCommand @updateParams;

                    WriteOperationSummary $mailPublicFolder $LocalizedStrings.UpdateOperationName $LocalizedStrings.CsvSuccessResult $updateProperties;

                    $validExternalEmailAddresses.Add($externalEmailAddress, $false);
                    $script:UpdatedPublicFoldersCount++;
               }

               catch
               {
                    WriteErrorSummary $mailPublicFolder $LocalizedStrings.UpdateOperationName $_.Exception.Message $updateProperties;
                    Write-Error $_;
               }

            }

            StoreExternalEmailAddressDomain $externalEmailAddress;
            WriteInfoMessage ($LocalizedStrings.DoneSyncingMailPublicFolder -f $alias);
            Write-Host "";
        }
    }

    else
    {
        WriteInfoMessage ($LocalizedStrings.NoMailPublicFoldersToSync);
        Write-Host "";
    }

    WriteInfoMessage ($LocalizedStrings.DeleteSyncMailPublicFolderTitle);

    $localMailPublicFolders = Get-MailPublicFolder -ErrorAction:SilentlyContinue -WarningAction:SilentlyContinue;

	$foldersToMailDisable = GetFoldersToMailDisable $localMailPublicFolders $validExternalEmailAddresses;
	if ($foldersToMailDisable -eq $null)
	{
		return;
	}
   
    $foldersToMailDisableFile = Join-Path $PWD "FoldersToMailDisable.txt"
    Set-Content -Path $foldersToMailDisableFile -Value $foldersToMailDisable      
    do 
    {
        $confirmation = Read-Host ($LocalizedStrings.FoldersToMailDisableConfirmation -f $foldersToMailDisableFile); 
    } 
    while ($confirmation -ne "Y" -and $confirmation -ne "y" -and $confirmation -ne "N" -and $confirmation -ne "n") 
    
    if ($confirmation -eq "N" -or $confirmation -eq "n") 
    { 
        return; 
    }
	
    foreach($syncPublicFolder in $localMailPublicFolders)
    {
        $localExternalEmailAddress = [String]::Empty;

        if ($syncPublicFolder.ExternalEmailAddress -ne $null)
        {
            $localExternalEmailAddress = RemoveSmtpPrefix $syncPublicFolder.ExternalEmailAddress.ToString();
        }

        if (-not $validExternalEmailAddresses.ContainsKey($localExternalEmailAddress))
        {
            WriteInfoMessage ($LocalizedStrings.DeletingSyncMailPublicFolder -f $syncPublicFolder);
            try
            {
                $deleteParams = @{};
                $deleteParams.Add("Identity", $syncPublicFolder);
                $deleteParams.Add("Confirm", $false);
                [string]$disableMailPublicFolder = (FormatCommand $script:DeletePublicFolderCommand $deleteParams);

                # Deleting sync mail public folder
                &$script:DeletePublicFolderCommand @deleteParams;

                WriteOperationSummary $syncPublicFolder $LocalizedStrings.DeleteOperationName $LocalizedStrings.CsvSuccessResult $disableMailPublicFolder;
                $script:RemovedPublicFoldersCount++;
            }
            catch
            {
                WriteErrorSummary $syncPublicFolder $LocalizedStrings.DeleteOperationName $_.Exception.Message $disableMailPublicFolder;
                Write-Error $_;
            }
        }
    }
}

################ DECLARING GLOBAL VARIABLES ################
$script:session = $null;

$script:csvSpecialChars = @("`r", "`n");
$script:csvEscapeChar = '"';
$script:csvFieldDelimiter = ',';
$script:separator = "`n`t";
$script:NewSyncMailPublicFolderCommand = "New-SyncMailPublicFolder";
$script:SetMailPublicFolderCommand = "Set-MailPublicFolder";
$script:DeletePublicFolderCommand = "Disable-MailPublicFolder";
$script:CreatedPublicFoldersCount = 0;
$script:UpdatedPublicFoldersCount = 0;
$script:RemovedPublicFoldersCount = 0;
$script:ExternalEmailAddressDomains = @{};
$script:SendConnectorToO365 = "Outbound to Office 365";
$script:OnMicrosoftDomain = ".onmicrosoft.com";
$script:SmtpPrefix = "smtp:";
$script:SmtpPrefixLength = $script:SmtpPrefix.Length;
[char[]]$script:proxyAddressSeparators = ':','@';

#load hashtable of localized string
Import-LocalizedData -BindingVariable LocalizedStrings -FileName SyncMailPublicFoldersCloudToOnprem.strings.psd1

#minimum supported exchange version to run this script
$minSupportedVersion = 15
################ END OF DECLARATION #########################

if (Test-Path $CsvSummaryFile)
{
    Remove-Item $CsvSummaryFile -Confirm:$Confirm -Force;
}

# Write the output CSV headers
$csvFile = New-Item -Path $CsvSummaryFile -ItemType File -Force -ErrorAction:Stop -Value ("#{0},{1},{2},{3},{4}`r`n" -f $LocalizedStrings.TimestampCsvHeader,
    $LocalizedStrings.IdentityCsvHeader,
    $LocalizedStrings.OperationCsvHeader,
    $LocalizedStrings.ResultCsvHeader,
    $LocalizedStrings.CommandCsvHeader);

$localServerVersion = (Get-ExchangeServer $env:COMPUTERNAME -ErrorAction:Stop).AdminDisplayVersion;
# This script can run from Exchange 2013 Management shell and above
if ($localServerVersion.Major -lt $minSupportedVersion)
{
    Write-Error ($LocalizedStrings.LocalServerVersionNotSupported -f $localServerVersion) -ErrorAction:Continue;
    Exit;
}

# Create a PSSession
WriteInfoMessage ($LocalizedStrings.CreatingRemoteSession);

InitializeExchangeOnlineRemoteSession;

# Get mail enabled public folders in cloud
WriteInfoMessage ($LocalizedStrings.StartedImportingMailPublicFolders);
Write-Host "";

$mailPublicFoldersEXO = GetRemoteMailPublicFolders;

# Create sync mail public folders in on-premise
SyncMailPublicFolders $mailPublicFoldersEXO;

Write-Host "";
WriteInfoMessage ($LocalizedStrings.CompletedImportingMailPublicFolders);
WriteInfoMessage ($LocalizedStrings.CompletedStatsCount -f  $script:CreatedPublicFoldersCount, $script:UpdatedPublicFoldersCount, $script:RemovedPublicFoldersCount);
Write-Host "";

if ((IsCentralizedTransportEnabled) -and $script:ExternalEmailAddressDomains.Count -gt 0)
{
    $sendConnectorDomains = GetSendConnectorDomains;

    if ($sendConnectorDomains -eq $null -or $sendConnectorDomains.Count -eq 0)
    {
        $domains = ConcatDomains $script:ExternalEmailAddressDomains.Keys;
        Write-Warning ($LocalizedStrings.VerifyConnectorAcceptedDomainsMessage -f $domains);
    }
    else
    {
        $missingDomains = GetMissingDomainsFromConnector $sendConnectorDomains;

        if ($missingDomains -ne $null -and $missingDomains.Count -gt 0)
        {
            $domains = ConcatDomains $missingDomains;
            Write-Warning ($LocalizedStrings.VerifyConnectorAcceptedDomainsMessage -f $domains);
        }
    }
}

# Terminate the PSSession
Disconnect-ExchangeOnline -Confirm:$false -ErrorAction SilentlyContinue;
# SIG # Begin signature block
# MIInwgYJKoZIhvcNAQcCoIInszCCJ68CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCDQrgKijLCWR8Ua
# VDwx2bMOfRbYu/woPiCbIt1eAws2BKCCDXYwggX0MIID3KADAgECAhMzAAADTrU8
# esGEb+srAAAAAANOMA0GCSqGSIb3DQEBCwUAMH4xCzAJBgNVBAYTAlVTMRMwEQYD
# VQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNy
# b3NvZnQgQ29ycG9yYXRpb24xKDAmBgNVBAMTH01pY3Jvc29mdCBDb2RlIFNpZ25p
# bmcgUENBIDIwMTEwHhcNMjMwMzE2MTg0MzI5WhcNMjQwMzE0MTg0MzI5WjB0MQsw
# CQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9u
# ZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMR4wHAYDVQQDExVNaWNy
# b3NvZnQgQ29ycG9yYXRpb24wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIB
# AQDdCKiNI6IBFWuvJUmf6WdOJqZmIwYs5G7AJD5UbcL6tsC+EBPDbr36pFGo1bsU
# p53nRyFYnncoMg8FK0d8jLlw0lgexDDr7gicf2zOBFWqfv/nSLwzJFNP5W03DF/1
# 1oZ12rSFqGlm+O46cRjTDFBpMRCZZGddZlRBjivby0eI1VgTD1TvAdfBYQe82fhm
# WQkYR/lWmAK+vW/1+bO7jHaxXTNCxLIBW07F8PBjUcwFxxyfbe2mHB4h1L4U0Ofa
# +HX/aREQ7SqYZz59sXM2ySOfvYyIjnqSO80NGBaz5DvzIG88J0+BNhOu2jl6Dfcq
# jYQs1H/PMSQIK6E7lXDXSpXzAgMBAAGjggFzMIIBbzAfBgNVHSUEGDAWBgorBgEE
# AYI3TAgBBggrBgEFBQcDAzAdBgNVHQ4EFgQUnMc7Zn/ukKBsBiWkwdNfsN5pdwAw
# RQYDVR0RBD4wPKQ6MDgxHjAcBgNVBAsTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEW
# MBQGA1UEBRMNMjMwMDEyKzUwMDUxNjAfBgNVHSMEGDAWgBRIbmTlUAXTgqoXNzci
# tW2oynUClTBUBgNVHR8ETTBLMEmgR6BFhkNodHRwOi8vd3d3Lm1pY3Jvc29mdC5j
# b20vcGtpb3BzL2NybC9NaWNDb2RTaWdQQ0EyMDExXzIwMTEtMDctMDguY3JsMGEG
# CCsGAQUFBwEBBFUwUzBRBggrBgEFBQcwAoZFaHR0cDovL3d3dy5taWNyb3NvZnQu
# Y29tL3BraW9wcy9jZXJ0cy9NaWNDb2RTaWdQQ0EyMDExXzIwMTEtMDctMDguY3J0
# MAwGA1UdEwEB/wQCMAAwDQYJKoZIhvcNAQELBQADggIBAD21v9pHoLdBSNlFAjmk
# mx4XxOZAPsVxxXbDyQv1+kGDe9XpgBnT1lXnx7JDpFMKBwAyIwdInmvhK9pGBa31
# TyeL3p7R2s0L8SABPPRJHAEk4NHpBXxHjm4TKjezAbSqqbgsy10Y7KApy+9UrKa2
# kGmsuASsk95PVm5vem7OmTs42vm0BJUU+JPQLg8Y/sdj3TtSfLYYZAaJwTAIgi7d
# hzn5hatLo7Dhz+4T+MrFd+6LUa2U3zr97QwzDthx+RP9/RZnur4inzSQsG5DCVIM
# pA1l2NWEA3KAca0tI2l6hQNYsaKL1kefdfHCrPxEry8onJjyGGv9YKoLv6AOO7Oh
# JEmbQlz/xksYG2N/JSOJ+QqYpGTEuYFYVWain7He6jgb41JbpOGKDdE/b+V2q/gX
# UgFe2gdwTpCDsvh8SMRoq1/BNXcr7iTAU38Vgr83iVtPYmFhZOVM0ULp/kKTVoir
# IpP2KCxT4OekOctt8grYnhJ16QMjmMv5o53hjNFXOxigkQWYzUO+6w50g0FAeFa8
# 5ugCCB6lXEk21FFB1FdIHpjSQf+LP/W2OV/HfhC3uTPgKbRtXo83TZYEudooyZ/A
# Vu08sibZ3MkGOJORLERNwKm2G7oqdOv4Qj8Z0JrGgMzj46NFKAxkLSpE5oHQYP1H
# tPx1lPfD7iNSbJsP6LiUHXH1MIIHejCCBWKgAwIBAgIKYQ6Q0gAAAAAAAzANBgkq
# hkiG9w0BAQsFADCBiDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24x
# EDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
# bjEyMDAGA1UEAxMpTWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9yaXR5
# IDIwMTEwHhcNMTEwNzA4MjA1OTA5WhcNMjYwNzA4MjEwOTA5WjB+MQswCQYDVQQG
# EwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwG
# A1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgwJgYDVQQDEx9NaWNyb3NvZnQg
# Q29kZSBTaWduaW5nIFBDQSAyMDExMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIIC
# CgKCAgEAq/D6chAcLq3YbqqCEE00uvK2WCGfQhsqa+laUKq4BjgaBEm6f8MMHt03
# a8YS2AvwOMKZBrDIOdUBFDFC04kNeWSHfpRgJGyvnkmc6Whe0t+bU7IKLMOv2akr
# rnoJr9eWWcpgGgXpZnboMlImEi/nqwhQz7NEt13YxC4Ddato88tt8zpcoRb0Rrrg
# OGSsbmQ1eKagYw8t00CT+OPeBw3VXHmlSSnnDb6gE3e+lD3v++MrWhAfTVYoonpy
# 4BI6t0le2O3tQ5GD2Xuye4Yb2T6xjF3oiU+EGvKhL1nkkDstrjNYxbc+/jLTswM9
# sbKvkjh+0p2ALPVOVpEhNSXDOW5kf1O6nA+tGSOEy/S6A4aN91/w0FK/jJSHvMAh
# dCVfGCi2zCcoOCWYOUo2z3yxkq4cI6epZuxhH2rhKEmdX4jiJV3TIUs+UsS1Vz8k
# A/DRelsv1SPjcF0PUUZ3s/gA4bysAoJf28AVs70b1FVL5zmhD+kjSbwYuER8ReTB
# w3J64HLnJN+/RpnF78IcV9uDjexNSTCnq47f7Fufr/zdsGbiwZeBe+3W7UvnSSmn
# Eyimp31ngOaKYnhfsi+E11ecXL93KCjx7W3DKI8sj0A3T8HhhUSJxAlMxdSlQy90
# lfdu+HggWCwTXWCVmj5PM4TasIgX3p5O9JawvEagbJjS4NaIjAsCAwEAAaOCAe0w
# ggHpMBAGCSsGAQQBgjcVAQQDAgEAMB0GA1UdDgQWBBRIbmTlUAXTgqoXNzcitW2o
# ynUClTAZBgkrBgEEAYI3FAIEDB4KAFMAdQBiAEMAQTALBgNVHQ8EBAMCAYYwDwYD
# VR0TAQH/BAUwAwEB/zAfBgNVHSMEGDAWgBRyLToCMZBDuRQFTuHqp8cx0SOJNDBa
# BgNVHR8EUzBRME+gTaBLhklodHRwOi8vY3JsLm1pY3Jvc29mdC5jb20vcGtpL2Ny
# bC9wcm9kdWN0cy9NaWNSb29DZXJBdXQyMDExXzIwMTFfMDNfMjIuY3JsMF4GCCsG
# AQUFBwEBBFIwUDBOBggrBgEFBQcwAoZCaHR0cDovL3d3dy5taWNyb3NvZnQuY29t
# L3BraS9jZXJ0cy9NaWNSb29DZXJBdXQyMDExXzIwMTFfMDNfMjIuY3J0MIGfBgNV
# HSAEgZcwgZQwgZEGCSsGAQQBgjcuAzCBgzA/BggrBgEFBQcCARYzaHR0cDovL3d3
# dy5taWNyb3NvZnQuY29tL3BraW9wcy9kb2NzL3ByaW1hcnljcHMuaHRtMEAGCCsG
# AQUFBwICMDQeMiAdAEwAZQBnAGEAbABfAHAAbwBsAGkAYwB5AF8AcwB0AGEAdABl
# AG0AZQBuAHQALiAdMA0GCSqGSIb3DQEBCwUAA4ICAQBn8oalmOBUeRou09h0ZyKb
# C5YR4WOSmUKWfdJ5DJDBZV8uLD74w3LRbYP+vj/oCso7v0epo/Np22O/IjWll11l
# hJB9i0ZQVdgMknzSGksc8zxCi1LQsP1r4z4HLimb5j0bpdS1HXeUOeLpZMlEPXh6
# I/MTfaaQdION9MsmAkYqwooQu6SpBQyb7Wj6aC6VoCo/KmtYSWMfCWluWpiW5IP0
# wI/zRive/DvQvTXvbiWu5a8n7dDd8w6vmSiXmE0OPQvyCInWH8MyGOLwxS3OW560
# STkKxgrCxq2u5bLZ2xWIUUVYODJxJxp/sfQn+N4sOiBpmLJZiWhub6e3dMNABQam
# ASooPoI/E01mC8CzTfXhj38cbxV9Rad25UAqZaPDXVJihsMdYzaXht/a8/jyFqGa
# J+HNpZfQ7l1jQeNbB5yHPgZ3BtEGsXUfFL5hYbXw3MYbBL7fQccOKO7eZS/sl/ah
# XJbYANahRr1Z85elCUtIEJmAH9AAKcWxm6U/RXceNcbSoqKfenoi+kiVH6v7RyOA
# 9Z74v2u3S5fi63V4GuzqN5l5GEv/1rMjaHXmr/r8i+sLgOppO6/8MO0ETI7f33Vt
# Y5E90Z1WTk+/gFcioXgRMiF670EKsT/7qMykXcGhiJtXcVZOSEXAQsmbdlsKgEhr
# /Xmfwb1tbWrJUnMTDXpQzTGCGaIwghmeAgEBMIGVMH4xCzAJBgNVBAYTAlVTMRMw
# EQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVN
# aWNyb3NvZnQgQ29ycG9yYXRpb24xKDAmBgNVBAMTH01pY3Jvc29mdCBDb2RlIFNp
# Z25pbmcgUENBIDIwMTECEzMAAANOtTx6wYRv6ysAAAAAA04wDQYJYIZIAWUDBAIB
# BQCgga4wGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEO
# MAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIEIPXHrWDL9Fbt5fY+dVg2z+7a
# jKlInkwFCBu+R+IS3UyLMEIGCisGAQQBgjcCAQwxNDAyoBSAEgBNAGkAYwByAG8A
# cwBvAGYAdKEagBhodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20wDQYJKoZIhvcNAQEB
# BQAEggEAsdg09UlmhSZGDn49UUaJBOg3bYhwIT0xhEUftn+xQ+VZzwqQ956ubj3M
# wnOvGfleFwgrUi/mOLE8sS2EIwERqRn1iE/NJXJWY0G8QSMxQjbQLzngsn4Tjb2r
# lVBJbpnxoY3dwKHpZRzlS7zuo5qUZoiWmTL25glWUVKNSjfjbOivV4hebN8cIFM2
# JSiQ4toumqzCEztodWZI/E+4JstUDA6UU9CH3JtT6Frw0605tRMbFrqDB9CuPgy0
# WRm3IqZ/AHyf8U83T+uem4zCAc6tW7IcD5ntuigzZ3FahGINDQbyFwvY/p9ltTV1
# 6sJv9/2O2FhCJCuUs3INA0MeUkPVAKGCFywwghcoBgorBgEEAYI3AwMBMYIXGDCC
# FxQGCSqGSIb3DQEHAqCCFwUwghcBAgEDMQ8wDQYJYIZIAWUDBAIBBQAwggFZBgsq
# hkiG9w0BCRABBKCCAUgEggFEMIIBQAIBAQYKKwYBBAGEWQoDATAxMA0GCWCGSAFl
# AwQCAQUABCCRB+/WjUpBCVDVrC7tN2cyIUD4xiOZ77FQfD08G/7SIwIGZGzuzYQZ
# GBMyMDIzMDUyNTAxNTc0MS40MTlaMASAAgH0oIHYpIHVMIHSMQswCQYDVQQGEwJV
# UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UE
# ChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMS0wKwYDVQQLEyRNaWNyb3NvZnQgSXJl
# bGFuZCBPcGVyYXRpb25zIExpbWl0ZWQxJjAkBgNVBAsTHVRoYWxlcyBUU1MgRVNO
# OjNCRDQtNEI4MC02OUMzMSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1TdGFtcCBT
# ZXJ2aWNloIIRezCCBycwggUPoAMCAQICEzMAAAG0+4AIRAXSLfoAAQAAAbQwDQYJ
# KoZIhvcNAQELBQAwfDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24x
# EDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
# bjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAwHhcNMjIw
# OTIwMjAyMjA5WhcNMjMxMjE0MjAyMjA5WjCB0jELMAkGA1UEBhMCVVMxEzARBgNV
# BAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jv
# c29mdCBDb3Jwb3JhdGlvbjEtMCsGA1UECxMkTWljcm9zb2Z0IElyZWxhbmQgT3Bl
# cmF0aW9ucyBMaW1pdGVkMSYwJAYDVQQLEx1UaGFsZXMgVFNTIEVTTjozQkQ0LTRC
# ODAtNjlDMzElMCMGA1UEAxMcTWljcm9zb2Z0IFRpbWUtU3RhbXAgU2VydmljZTCC
# AiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBALRHpp5lBzJCH7zortuyvOmW
# 8FoZLBsFe9g5dbhnaq9qSpvpn86E/mJ4JKvWixH/lw7QA8gPtiiGVNIjvFhu/XiY
# 889vX5WaQSmyoPMZdj9zvXa5XrkMN05zXzTePkCIIzF6RN7cTxezOyESymTIjrdx
# X5BVlZolyQAOxNziMCYKYYNPbYd0786fDE/PhzrRt23a0Xf8trvFa0LEEy2YlcE2
# eqg2CjU/D0GZe8Ra0kjt0M12vdS4qWZ2Dpd7IhiQwnntQWu19Ytd3UBR8SpeRX+C
# cw3bjgWfOXtla6chctWt2shlMwayMOfY4TG4yMPWFXELfZFFp7cgpjZNeVsmwkvo
# V6RAwy1Y9V+VvbJ5qFtartN/rp6a0I1kGlbjuwX3L0HTVXcikqgHistXk9h3HOZ9
# WgFXlxZurG1SZmcz0BEEdya+1vGHE45KguYU9qq2LiHGBjn9z4+DqnV5tUKobsLb
# JMb4r+8st2fj8SacSsftnusxkWqEJiJS34P2uNlzVR03+ls6+ZO0NcO79LgP7BbI
# MipiOx8yh19PMQw0piaKFwOW7Q+gdJcfy6rOkG+CrYZwOzdiBHSebIzCIch2cAa+
# 38w7JFP/koKdlJ36qzdVXWv4G/qZpWycIvDKYbxJWM40+z2Stg5uHqK3I8e09kFX
# txCHpS7hm8c8m25WaEU5AgMBAAGjggFJMIIBRTAdBgNVHQ4EFgQUy0SF5fGUuDqc
# uxIot07eOMwy2X4wHwYDVR0jBBgwFoAUn6cVXQBeYl2D9OXSZacbUzUZ6XIwXwYD
# VR0fBFgwVjBUoFKgUIZOaHR0cDovL3d3dy5taWNyb3NvZnQuY29tL3BraW9wcy9j
# cmwvTWljcm9zb2Z0JTIwVGltZS1TdGFtcCUyMFBDQSUyMDIwMTAoMSkuY3JsMGwG
# CCsGAQUFBwEBBGAwXjBcBggrBgEFBQcwAoZQaHR0cDovL3d3dy5taWNyb3NvZnQu
# Y29tL3BraW9wcy9jZXJ0cy9NaWNyb3NvZnQlMjBUaW1lLVN0YW1wJTIwUENBJTIw
# MjAxMCgxKS5jcnQwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEFBQcD
# CDAOBgNVHQ8BAf8EBAMCB4AwDQYJKoZIhvcNAQELBQADggIBABLRDwWMKbeCYqEq
# tI6Bs8KmF+kqDR+2G6qYAK3ZZ63bert7pCkRJbihFaktl2o18cdFJFxnOF4vXadm
# 0sabskJ05KviEMJIO6dXSq8AGtr3Zmjc895q0mnlBLuNMgk4R8KrkJMHqBuHqkUW
# XtfTrVUpgwzQt2UOiINKs+/b4r14MuXRVpOJ6cQOS8UhkeMAWl2iLlYaBGtOr3f/
# f9mLEPfWwoke0sSUbdV60OZCRh1ItBYYM9efKr14H5qu6jan6n00prEEa7W3uGb/
# 1/qj6P5emnvkqy5HI0X69DjVdLxVbjSsegm/dA+S4DaXPcfFf6iBxK/iV21l1upg
# EVVajUApl5VR40wY4XF8EpmnUdTqLXDf7CqdhDjPST2K/OjvWPyQGQvc7oPapYyk
# 66GU32AOyyHXJj6+vbtRUg/+ory+h0R2Xf5NhC+xbWcMzXEUXRRf1YKZDsRyH6r4
# 12pm8KDKE/r7Rk7aoKK7oYUpNGzNRf6QaYv5z2bVTSxkzWivFrepLHGwvRun9PYM
# /8AQSTgZr0yzzjk/97WghkqCaAwAVpyvg3uaYnuCl/AccSkGyb8c+70bFSeUephs
# fgb2r+QI7Mb2WcOnkJpCNLz0XJMS/UwlQn1ktLsiCpsqOk3aLJ2wTv6LK3u69I0v
# QB/LKRKlZYRXKUDXzoPwr3UtsTVTMIIHcTCCBVmgAwIBAgITMwAAABXF52ueAptJ
# mQAAAAAAFTANBgkqhkiG9w0BAQsFADCBiDELMAkGA1UEBhMCVVMxEzARBgNVBAgT
# Cldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29m
# dCBDb3Jwb3JhdGlvbjEyMDAGA1UEAxMpTWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNh
# dGUgQXV0aG9yaXR5IDIwMTAwHhcNMjEwOTMwMTgyMjI1WhcNMzAwOTMwMTgzMjI1
# WjB8MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
# UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSYwJAYDVQQD
# Ex1NaWNyb3NvZnQgVGltZS1TdGFtcCBQQ0EgMjAxMDCCAiIwDQYJKoZIhvcNAQEB
# BQADggIPADCCAgoCggIBAOThpkzntHIhC3miy9ckeb0O1YLT/e6cBwfSqWxOdcjK
# NVf2AX9sSuDivbk+F2Az/1xPx2b3lVNxWuJ+Slr+uDZnhUYjDLWNE893MsAQGOhg
# fWpSg0S3po5GawcU88V29YZQ3MFEyHFcUTE3oAo4bo3t1w/YJlN8OWECesSq/XJp
# rx2rrPY2vjUmZNqYO7oaezOtgFt+jBAcnVL+tuhiJdxqD89d9P6OU8/W7IVWTe/d
# vI2k45GPsjksUZzpcGkNyjYtcI4xyDUoveO0hyTD4MmPfrVUj9z6BVWYbWg7mka9
# 7aSueik3rMvrg0XnRm7KMtXAhjBcTyziYrLNueKNiOSWrAFKu75xqRdbZ2De+JKR
# Hh09/SDPc31BmkZ1zcRfNN0Sidb9pSB9fvzZnkXftnIv231fgLrbqn427DZM9itu
# qBJR6L8FA6PRc6ZNN3SUHDSCD/AQ8rdHGO2n6Jl8P0zbr17C89XYcz1DTsEzOUyO
# ArxCaC4Q6oRRRuLRvWoYWmEBc8pnol7XKHYC4jMYctenIPDC+hIK12NvDMk2ZItb
# oKaDIV1fMHSRlJTYuVD5C4lh8zYGNRiER9vcG9H9stQcxWv2XFJRXRLbJbqvUAV6
# bMURHXLvjflSxIUXk8A8FdsaN8cIFRg/eKtFtvUeh17aj54WcmnGrnu3tz5q4i6t
# AgMBAAGjggHdMIIB2TASBgkrBgEEAYI3FQEEBQIDAQABMCMGCSsGAQQBgjcVAgQW
# BBQqp1L+ZMSavoKRPEY1Kc8Q/y8E7jAdBgNVHQ4EFgQUn6cVXQBeYl2D9OXSZacb
# UzUZ6XIwXAYDVR0gBFUwUzBRBgwrBgEEAYI3TIN9AQEwQTA/BggrBgEFBQcCARYz
# aHR0cDovL3d3dy5taWNyb3NvZnQuY29tL3BraW9wcy9Eb2NzL1JlcG9zaXRvcnku
# aHRtMBMGA1UdJQQMMAoGCCsGAQUFBwMIMBkGCSsGAQQBgjcUAgQMHgoAUwB1AGIA
# QwBBMAsGA1UdDwQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MB8GA1UdIwQYMBaAFNX2
# VsuP6KJcYmjRPZSQW9fOmhjEMFYGA1UdHwRPME0wS6BJoEeGRWh0dHA6Ly9jcmwu
# bWljcm9zb2Z0LmNvbS9wa2kvY3JsL3Byb2R1Y3RzL01pY1Jvb0NlckF1dF8yMDEw
# LTA2LTIzLmNybDBaBggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKGPmh0dHA6Ly93
# d3cubWljcm9zb2Z0LmNvbS9wa2kvY2VydHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYt
# MjMuY3J0MA0GCSqGSIb3DQEBCwUAA4ICAQCdVX38Kq3hLB9nATEkW+Geckv8qW/q
# XBS2Pk5HZHixBpOXPTEztTnXwnE2P9pkbHzQdTltuw8x5MKP+2zRoZQYIu7pZmc6
# U03dmLq2HnjYNi6cqYJWAAOwBb6J6Gngugnue99qb74py27YP0h1AdkY3m2CDPVt
# I1TkeFN1JFe53Z/zjj3G82jfZfakVqr3lbYoVSfQJL1AoL8ZthISEV09J+BAljis
# 9/kpicO8F7BUhUKz/AyeixmJ5/ALaoHCgRlCGVJ1ijbCHcNhcy4sa3tuPywJeBTp
# kbKpW99Jo3QMvOyRgNI95ko+ZjtPu4b6MhrZlvSP9pEB9s7GdP32THJvEKt1MMU0
# sHrYUP4KWN1APMdUbZ1jdEgssU5HLcEUBHG/ZPkkvnNtyo4JvbMBV0lUZNlz138e
# W0QBjloZkWsNn6Qo3GcZKCS6OEuabvshVGtqRRFHqfG3rsjoiV5PndLQTHa1V1QJ
# sWkBRH58oWFsc/4Ku+xBZj1p/cvBQUl+fpO+y/g75LcVv7TOPqUxUYS8vwLBgqJ7
# Fx0ViY1w/ue10CgaiQuPNtq6TPmb/wrpNPgkNWcr4A245oyZ1uEi6vAnQj0llOZ0
# dFtq0Z4+7X6gMTN9vMvpe784cETRkPHIqzqKOghif9lwY1NNje6CbaUFEMFxBmoQ
# tB1VM1izoXBm8qGCAtcwggJAAgEBMIIBAKGB2KSB1TCB0jELMAkGA1UEBhMCVVMx
# EzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoT
# FU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEtMCsGA1UECxMkTWljcm9zb2Z0IElyZWxh
# bmQgT3BlcmF0aW9ucyBMaW1pdGVkMSYwJAYDVQQLEx1UaGFsZXMgVFNTIEVTTjoz
# QkQ0LTRCODAtNjlDMzElMCMGA1UEAxMcTWljcm9zb2Z0IFRpbWUtU3RhbXAgU2Vy
# dmljZaIjCgEBMAcGBSsOAwIaAxUAZZzYkPObl/ZzeCkSbf4B5CceCQiggYMwgYCk
# fjB8MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
# UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSYwJAYDVQQD
# Ex1NaWNyb3NvZnQgVGltZS1TdGFtcCBQQ0EgMjAxMDANBgkqhkiG9w0BAQUFAAIF
# AOgYvncwIhgPMjAyMzA1MjUwMDQ4NTVaGA8yMDIzMDUyNjAwNDg1NVowdzA9Bgor
# BgEEAYRZCgQBMS8wLTAKAgUA6Bi+dwIBADAKAgEAAgIXMgIB/zAHAgEAAgIRzDAK
# AgUA6BoP9wIBADA2BgorBgEEAYRZCgQCMSgwJjAMBgorBgEEAYRZCgMCoAowCAIB
# AAIDB6EgoQowCAIBAAIDAYagMA0GCSqGSIb3DQEBBQUAA4GBAC5m7zQUCLic65dJ
# j+cASHWg8ijdVW8GOevDr/HslDR0Om6dhp4qD/N4SeqknlVBNTmCpEJikAHXdFSl
# eZ+xUPMhha/ooa1jyEJS+mGk3Jin94IdATPKjQHq1Rvp1rs3ug93Km2iPNNpb2ua
# 0si23FCoS4E6mybwhYAlGtLt9PKsMYIEDTCCBAkCAQEwgZMwfDELMAkGA1UEBhMC
# VVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNV
# BAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRp
# bWUtU3RhbXAgUENBIDIwMTACEzMAAAG0+4AIRAXSLfoAAQAAAbQwDQYJYIZIAWUD
# BAIBBQCgggFKMBoGCSqGSIb3DQEJAzENBgsqhkiG9w0BCRABBDAvBgkqhkiG9w0B
# CQQxIgQgCZv5MEM2QmW4Y3K3jT5FqyKHEVq6hBXKlu4Q0HkTa2AwgfoGCyqGSIb3
# DQEJEAIvMYHqMIHnMIHkMIG9BCDTyPd75qMwcAZRcb36/6xJa3hT0eLse71ysdp4
# twH3BjCBmDCBgKR+MHwxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9u
# MRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRp
# b24xJjAkBgNVBAMTHU1pY3Jvc29mdCBUaW1lLVN0YW1wIFBDQSAyMDEwAhMzAAAB
# tPuACEQF0i36AAEAAAG0MCIEIHA3WGWcoCZ873qgE3hu/cWTA87DS3C6ijR/bpiU
# SEtcMA0GCSqGSIb3DQEBCwUABIICABOTqvpBK5w2VyZvSHXNaHFDinto337SbFjF
# 1FMeVcFiELTHstQr4pNvLMhd7ctfLIEo7IZGQwEgG5QrbhDwTrf/O8sMGnfBxRMc
# ftNI8vaF0+dbNxh2iWJw+6f/XvYjrJqQxw9CiCILRjfuYZ3t2Ue7Mz+arnuR7RrP
# fY58s2YulPCPmBngZvy0bQvSf34MvX97eFO2lWmL7nX6hcCzzBVBw/nYc7DBYTk7
# n7irjgcfpHf4OBGEnt6Oxj2EALsYj3MEQKKsvWD4v+lrhoeLYHZYQyQTofX+20Tc
# SOP0KNan56pq3LXRueNKax+3sFJUkN2NUZJzvNbUY01WwQI2BfLu6vseWl0oP27c
# /QbBPKKFQfqxqtD7oS9RlQZ7g+eY/jYVPUaUduuK3MA+F/gAhDv71EfxAjkCN8Yp
# 56UBX9sqfIRuQk9R/5+DpOmvli/bzbhum/Xz/DbCUDuNTkxLBJpzMimj6MOHJeBb
# 4ffmnfVbzRTeteG5gS6PWA2Y5yoZxuo9w1l3DDs+rm9pcF8RIquGcXwrNQ9OYuDj
# uZ/ve6/TjtPpaLXz0BHZb21qrsrR8VHdLWMVq9BlsRaZGTxvn7y+9QHs5/WURCz/
# VRXkKW5lIHeHyick0OwpucpJLBfkADn6S19+WQwSitBi754GNrFfKCNyngZNJLYz
# 3v8ox+s2
# SIG # End signature block
